<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
    xmlns:as="http://www.oasis-open.org/committees/cam" xmlns:camed="http://jcam.org.uk/editor"
    xmlns:look="http://jcam.org.uk/LookupLists" xmlns:uuid="java:java.util.UUID"
    xmlns:xmi="http://schema.omg.org/spec/XMI/2.1" xmlns:uml="http://schema.omg.org/spec/UML/2.1.2"
    xmlns:as-xsl="http://www.oasis-open.org/committees/cam/xslt"
    xmlns:UML="org.omg.xmi.namespace.UML"
    exclude-result-prefixes="xs xsd xd uuid xmi uml UML as as-xsl look camed" version="2.0">
    <xd:doc scope="stylesheet">
        <xd:desc>
            <xd:p><xd:b>Created on:</xd:b> March 4, 2011</xd:p>
            <xd:p><xd:b>Author:</xd:b> DRRW</xd:p>
            <xd:p/>
        </xd:desc>
    </xd:doc>

    <!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/
            Author: David RR Webber, March, 2011 -->
    <!--
            Purpose  : Read in Dictionary file (compiled or uncompiled) - inspect header/dictionary details; return details back to Java via stdout. 
            Original : 03/09/2012
            Initials : drrw
            CAM ver    : 1.1 
            Description : Part of a set of tools to provide open source tools for using OASIS CAM dictionaries.
            Updated  - Inits - Change
            03/12/12   drrw    Initial release
            03/13/12   drrw    Add namespaces
            03/15/12   drrw    Exclude namespaces for xml, xsd, as and camedit
            11/29/12   drrw    Change file handling to allow non-file targets (http: et al)
            12/10/12   drrw    Revise file handling of targets and prefixes
           
        -->

    <xsl:param name="DictionaryFile">file:/temp/test/dictionary-file.xml</xsl:param>
    
    <!-- none, cardinality, content, description -->

    <xsl:variable name="xslt-version">1.05</xsl:variable>
    <xsl:variable name="debug">0</xsl:variable>
    <xsl:variable name="excludeNS">|as|xml|xsd|xs|xmi|camed|</xsl:variable>

    <xsl:variable name="DictionaryFilePath">
        <xsl:choose>
            <xsl:when test="contains($DictionaryFile,'://')"><xsl:value-of select="normalize-space(translate($DictionaryFile,'\','/'))"/></xsl:when>
            <xsl:when test="starts-with($DictionaryFile,'/')">file://<xsl:value-of select="translate($DictionaryFile,'\','/')"/></xsl:when>
            <xsl:when test="starts-with($DictionaryFile,'./')"><xsl:value-of select="translate($DictionaryFile,'\','/')"/></xsl:when>
            <xsl:when test="starts-with($DictionaryFile,'../')"><xsl:value-of select="translate($DictionaryFile,'\','/')"/></xsl:when>
            <xsl:when test="contains($DictionaryFile,'\')"><xsl:value-of select="if(starts-with($DictionaryFile,'file:')) then translate($DictionaryFile,'\','/') else concat('file:///',translate($DictionaryFile,'\','/'))"/></xsl:when>
            <xsl:otherwise><xsl:value-of select="$DictionaryFile"/></xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    <xsl:variable name="LookupDictionary" select="document($DictionaryFilePath)"/>
    
    <xsl:variable name="EpochSeconds">
        <!-- xsl:sequence select="current-dateTime()"/ -->
        <xsl:sequence
            select="( current-dateTime() - xs:dateTime('1970-01-01T00:00:00') )
            div xs:dayTimeDuration('PT1S') * 1000"
        />
    </xsl:variable>

     <xsl:variable name="dictionary">
            <components shortname="{@shortname}" namespaces="{@namespaces}">
                <!-- xsl:copy-of select="document($thisFile)"/ -->
                <xsl:variable name="tempDictionary" select="$LookupDictionary"/>
                <ITEMSlookup>
                    <xsl:comment><xsl:value-of select="$tempDictionary/ITEMSlookup/comment()"/></xsl:comment>
                    <!-- now unpack the context -->
                    <xsl:for-each select="$tempDictionary//Property">
                        <xsl:variable name="propertyDetails" select="."/>
                        <xsl:variable name="uniqueContext" select="tokenize(./@context,'&#xA;')"/>
                        <xsl:for-each select="$uniqueContext">
                            <xsl:if test="string-length(.) &gt; 0">
                                <xsl:element name="Property">
                                    <xsl:for-each select="$propertyDetails/@*">
                                        <xsl:choose>
                                            <xsl:when test="name()='context'"/>
                                            <!-- skip existing context -->
                                            <xsl:when test="name()='group'"/>
                                            <!-- skip  -->
                                            <xsl:when test="name()='where'"/>
                                            <!-- skip  -->
                                            <xsl:when test="name()='notes'"/>
                                            <!-- skip  -->
                                            <xsl:when test="name()='language'"/>
                                            <!-- skip  -->
                                            <xsl:when test="name()='anno'"/>
                                            <!-- skip  -->
                                            <xsl:otherwise>
                                                <xsl:attribute name="{name()}" select="."/>
                                            </xsl:otherwise>
                                        </xsl:choose>
                                    </xsl:for-each>
                                    <xsl:attribute name="context" select="."/>
                                </xsl:element>
                            </xsl:if>
                        </xsl:for-each>
                    </xsl:for-each>
                </ITEMSlookup>
            </components>
    </xsl:variable>

    <xsl:output method="xml" indent="yes" omit-xml-declaration="yes"
        exclude-result-prefixes="xd xs xsd look as camed"/>

    <xsl:template match="/">
        <xsl:message>Dictionary Util starting <xsl:value-of
                select="dateTime(current-date(),current-time())"/></xsl:message>
        <xsl:message>Version: <xsl:value-of select="$xslt-version"/></xsl:message>
        <xsl:message>Dictionary File Location: <xsl:value-of select="$DictionaryFilePath"/></xsl:message>
        <xsl:variable name="recordsCount" select="count($LookupDictionary//ITEMSlookup)"/>
        <xsl:variable name="dictionaryNSlist" select="$LookupDictionary/*/namespace::*"/>
        
        <xsl:choose>
            <xsl:when test="not(doc-available($DictionaryFilePath))">
                <xsl:message terminate="yes">ERROR: Dictionary input file is not available!</xsl:message>
            </xsl:when>
            <xsl:when test="count($LookupDictionary//Dictionary) &lt; 1 and($recordsCount &lt; 1)">
                <xsl:message terminate="yes">ERROR: Dictionary input file is invalid!</xsl:message>
            </xsl:when>
            <xsl:otherwise/>
        </xsl:choose>
                           
        <xsl:choose>
            <xsl:when test="$LookupDictionary//Dictionary">
                <!-- Process the compiled format of the dictionary -->
                <!-- xsl:comment>Dictionary <xsl:value-of select="$LookupDictionary/Dictionary/@name"/></xsl:comment -->
                <xsl:message>Collecting namespaces for compiled <xsl:value-of select="$LookupDictionary/Dictionary/@name"/></xsl:message>
                <xsl:element name="Dictionary">
                    <xsl:attribute name="name" select="$LookupDictionary/Dictionary/@name"/>
                    <xsl:attribute name="description" select="$LookupDictionary/Dictionary/@description"/>
                    <xsl:attribute name="item" select="$LookupDictionary/Dictionary/@item"/>
                    <xsl:attribute name="shortname" select="$LookupDictionary/Dictionary/@shortname"/>
                    <xsl:attribute name="namespaces" select="$LookupDictionary/Dictionary/@namespaces"/>
                    <xsl:element name="as:Namespaces">
                        <xsl:for-each select="$dictionaryNSlist">
                            <xsl:variable name="nsPrefix" select="name(.)"/>
                            <xsl:choose>
                                <xsl:when test="contains($excludeNS,concat('|',$nsPrefix,'|'))"/><!-- skip matching namespace -->
                                <xsl:otherwise>
                                    <xsl:element name="as:namespace">
                                        <xsl:attribute name="prefix" select="$nsPrefix"/>
                                        <xsl:value-of select="."/>
                                    </xsl:element>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:for-each>
                    </xsl:element>
                </xsl:element>
            </xsl:when>
            <xsl:otherwise><!-- Process uncompiled dictionary -->
                <xsl:variable name="dictionaryName" select="$LookupDictionary/ITEMSlookup/Property[1]/@item"/>
                <xsl:message>Collecting namespaces for <xsl:value-of select="$dictionaryName"/></xsl:message>
                <xsl:variable name="ns-list">
                    <xsl:for-each select="$dictionary//ITEMSlookup/Property[contains(@item,':')]/@item"><xsl:value-of select="substring-before(.,':')"/>|</xsl:for-each>
                </xsl:variable>
                <xsl:variable name="ns-uniques" select="distinct-values(tokenize($ns-list,'\|'))"/>
                <!-- now figure out the default namespace and put that first in the list -->
                <xsl:variable name="ns-default" select="if(contains(/ITEMSlookup/Property[1]/@item,':')) then '' else ','"/>
                <xsl:variable name="ns-parm"><xsl:value-of select="$ns-default"/><xsl:for-each select="$ns-uniques"><xsl:if test="not(. = '')"><xsl:if test="position() &gt; 1">,</xsl:if><xsl:value-of select="."/></xsl:if></xsl:for-each></xsl:variable>
                
                <xsl:element name="Dictionary">
                    <xsl:attribute name="name" select="$dictionaryName"/>
                    <xsl:attribute name="description" select="concat('Dictionary ',$dictionaryName)"/>
                    <xsl:attribute name="item" select="concat('//camed:dictionary[@shortname=','''',$dictionaryName,'''',']')"/>
                    <xsl:attribute name="shortname" select="$dictionaryName"/>
                    <xsl:attribute name="namespaces" select="$ns-parm"/>
                    <xsl:element name="as:Namespaces">
                        <xsl:for-each select="$dictionaryNSlist">
                            <xsl:variable name="nsPrefix" select="name(.)"/>
                            <xsl:choose>
                                <xsl:when test="contains($excludeNS,concat('|',$nsPrefix,'|'))"/><!-- skip matching namespace -->
                                <xsl:otherwise>
                                 <xsl:element name="as:namespace">
                                     <xsl:attribute name="prefix" select="$nsPrefix"/>
                                     <xsl:value-of select="."/>
                                 </xsl:element>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:for-each>
                    </xsl:element>
                </xsl:element>
            </xsl:otherwise>
        </xsl:choose>
 
        <xsl:message>Namespaces Processing Completed</xsl:message>

        <xsl:message>Finished Dictionary Util</xsl:message>
    </xsl:template>
    
</xsl:stylesheet>
